@@ -1,8 +1,8 @@
-# $Header: /usr/local/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v 1.5 2006/11/26 06:47:43 matisse Exp $
-# $Revision: 1.5 $
+# $Header: /usr/local/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v 1.8 2006/11/27 06:06:00 matisse Exp $
+# $Revision: 1.8 $
 # $Author: matisse $
 # $Source: /usr/local/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v $
-# $Date: 2006/11/26 06:47:43 $
+# $Date: 2006/11/27 06:06:00 $
 ###############################################################################
 
 package Perl::Metrics::Simple::Analysis::File;
@@ -16,7 +16,7 @@ use Perl::Metrics::Simple::Analysis;
 use PPI;
 use Readonly;
 
-our $VERSION = '0.021';
+our $VERSION = '0.022';
 
 Readonly::Scalar my $ALL_NEWLINES_REGEX => qr/ ( \n ) /xm;
 Readonly::Array our @LOGIC_OPERATORS    =>
@@ -52,12 +52,12 @@ sub _init {
         confess "Path '$path' is missing or not readable!";
     }
 
-    my $document = PPI::Document->new( $path, readonly => 1 );
+    my $document = _make_pruned_document($path);
     if ( !defined $document ) {
         cluck "Could not make a PPI document from '$path'";
         return;
     }
-    $document->index_locations();
+
     my $packages = _get_packages($document);
 
     my @sub_analysis = ();
@@ -73,6 +73,16 @@ sub _init {
     return $self;
 }
 
+
+sub _make_pruned_document {
+    my $path = shift;
+    my $document = PPI::Document->new( $path);
+    $document= _prune_comments_and_pod($document);
+    $document->index_locations();
+    $document->readonly(1);
+    return $document
+}
+
 sub all_counts {
     my $self       = shift;
     my $stats_hash = {
@@ -107,9 +117,24 @@ sub analyze_main {
 
 sub get_node_length {
     my ( $self, $node ) = @_;
+    eval {
+        $node = _prune_comments_and_pod($node);
+    };
     my $string = $node->content;
+    return 0 if ( ! length $string );
+    $string =~ s/ \s+ \n /\n/msxg;
+    $string =~ s/ \A \s+ //msx;
     my @newlines = ( $string =~ /$ALL_NEWLINES_REGEX/mxg );
-    return scalar @newlines + 1;
+    my $line_count = scalar @newlines;
+
+    # if the string is not empty and the last character is not a newline then add 1
+    if ( length $string ) {
+        my $last_char = substr $string, -1, 1;
+        if ( $last_char ne "\n" ) {
+            $line_count++;
+        }
+    }
+    return $line_count;
 }
 
 sub path {
@@ -140,19 +165,32 @@ sub lines {
 sub measure_complexity {
     my $self  = shift;
     my $elem  = shift;
-    my $count = 1;
+    
+    my $complexity_count = 0;
+
+    eval {
+        $elem->prune('PPI::Token::Comment');
+        $elem->prune('PPI::Token::Pod');
+    };
+    if ( $EVAL_ERROR ) {
+        return $complexity_count;
+    }
+
+    if ( length "$elem" ) {
+        $complexity_count++;
+    }
 
     # Count up all the logic keywords, weed out hash keys
-    my $keywords_ref = $elem->find('PPI::Token::Word');
+    my $keywords_ref = $elem->find('PPI::Token::Word') || [];
     my @filtered = grep { !is_hash_key($_) } @{$keywords_ref};
-    $count += grep { exists $LOGIC_KEYWORDS{$_} } @filtered;
+    $complexity_count += grep { exists $LOGIC_KEYWORDS{$_} } @filtered;
 
     # Count up all the logic operators
     my $operators_ref = $elem->find('PPI::Token::Operator');
     if ($operators_ref) {
-        $count += grep { exists $LOGIC_OPERATORS{$_} } @{$operators_ref};
+        $complexity_count += grep { exists $LOGIC_OPERATORS{$_} } @{$operators_ref};
     }
-    return $count;
+    return $complexity_count;
 }
 
 sub _get_packages {
@@ -228,8 +266,18 @@ sub is_hash_key {
     return $is_hash_key;
 }
 
+sub _prune_comments_and_pod {
+    my $document = shift;
+
+    $document->prune('PPI::Token::Comment');
+    $document->prune('PPI::Token::Pod');
+    return $document;
+}
+
 1;
 
+
+
 __END__
 
 =head1 NAME
@@ -292,26 +340,37 @@ and returns a hashref with information about the 'main' (non-subroutine)
 portions of the document:
 
   {
-    lines             => $lines,      # line count outside subs
-    mccabe_complexity => $complexity, # cyclomatic complexity of all non-sub areas
+    lines             => $lines,      # Line count outside subs. Skips comments and pod.
+    mccabe_complexity => $complexity, # Cyclomatic complexity of all non-sub areas
   };
 
 =head2 get_node_length
 
 Takes a B<PPI> node and returns a count of the newlines it
-contains. B<PPI> norm,ailizes line endings to newlines so
-CR/LF, CR and LF all come out the same.
-
+contains. B<PPI> normalizes line endings to newlines so
+CR/LF, CR and LF all come out the same. The line counts reported by
+the various methods in this class all B<exclude> blank lines,
+comment lines and pod
+(the B<PPI> document is pruned before counting.)
 
 =head2 lines
 
+Total non-blank, non-comment/pod lines.
+
 =head2 main_stats
 
+Returns the hashref generated by I<analyze_main>.
+
 =head2 measure_complexity
 
-Takes a B<PPI> element and measures the Mccabe Complexity
-(aka Cyclomatic Complexity) of the code. The complexity count is 1 plus
-1 for each logic keyword or operator:
+Takes a B<PPI> element and measures the McCabe Complexity
+(aka Cyclomatic Complexity) of the code.  McCabe Complexity
+is basically a count of how many paths there are through the code.
+We use a simplified method for count this, which ignores things like
+the possibility that a 'use' statement with throw an exception.
+
+The complexity count is 1
+for any non-comment/pod code plus 1 for each logic keyword or operator:
 
 Logic operators:
 
@@ -321,6 +380,19 @@ Logic keywords:
 
  for foreach goto if else elsif last next unless until while
 
+Example of  complexity 1:
+
+   # Only one  path through the code.
+   use Foo;
+   print "Hello world.\n";
+   exit;
+
+Example of complexity 2:
+
+   # There are two possible paths through this code:
+   if ( $a ) {
+   }
+
 =head2 packages
 
 Arrayref of unique packages found in the file.
